#include "ggap/searchbox.h"
#include "ui_searchbox.h"
#include <QtGui>

using namespace ggap;

class ggap::SearchBoxPrivate : public Ui_SearchBox {
    M_DECLARE_PUBLIC(SearchBox)

public:
    QTextEdit *text;

    SearchBoxPrivate(SearchBox *pub) :
        pub(pub),
        text(0)
    {
    }
};

SearchBox::SearchBox(QWidget *parent) :
    QFrame(parent),
    impl(this)
{
    impl->setupUi(this);
    connect(impl->searchButtonNext, SIGNAL(clicked()), SLOT(findNext()));
    connect(impl->searchButtonPrev, SIGNAL(clicked()), SLOT(findPrevious()));
    connect(impl->searchButtonClose, SIGNAL(clicked()), SLOT(cancelFind()));
    connect(impl->searchEntry, SIGNAL(textEdited(QString)), SLOT(entryTextEdited(QString)));
    impl->searchEntry->installEventFilter(this);
}

SearchBox::~SearchBox()
{
}

void SearchBox::setTextWidget(QTextEdit *text)
{
    impl->text = text;
}

void SearchBox::find()
{
    show();

    QString text = impl->text->textCursor().selectedText();
    if (!text.isEmpty() && !text.contains(QChar(0x2029)))
        impl->searchEntry->setText(text);

    impl->searchEntry->setFocus(Qt::OtherFocusReason);
    impl->searchEntry->selectAll();
}

void SearchBox::cancelFind()
{
    bool had_focus = impl->searchEntry->hasFocus();
    hide();
    if (had_focus)
        impl->text->setFocus(Qt::OtherFocusReason);
}

void SearchBox::doFind(const QString &text, bool forward)
{
    QTextDocument::FindFlags flags = 0;
    Qt::CaseSensitivity cs = Qt::CaseInsensitive;

    if (!forward)
        flags |= QTextDocument::FindBackward;
    if (impl->searchCheckCaseSensitive->isChecked())
    {
        flags |= QTextDocument::FindCaseSensitively;
        cs = Qt::CaseSensitive;
    }
    if (impl->searchCheckWholeWords->isChecked())
        flags |= QTextDocument::FindWholeWords;

    int start_pos = -1;
    QTextCursor cr = impl->text->textCursor();

    if (cr.hasSelection())
    {
        QString s = cr.selectedText();
        if (forward)
        {
            if (s.startsWith(text) && QString::compare(s, text, cs) != 0)
            {
                cr.setPosition(cr.selectionStart());
                cr.setPosition(cr.position() + text.length(), QTextCursor::KeepAnchor);
                impl->text->setTextCursor(cr);
                impl->text->ensureCursorVisible();
                return;
            }

            if (text.startsWith(s, cs) && QString::compare(s, text, cs) != 0)
                start_pos = cr.selectionStart();
            else
                start_pos = cr.selectionStart() + 1;
        }
        else
        {
            if (s.endsWith(text) && QString::compare(s, text, cs) != 0)
            {
                cr.setPosition(cr.selectionEnd());
                cr.setPosition(cr.position() - text.length(), QTextCursor::KeepAnchor);
                impl->text->setTextCursor(cr);
                impl->text->ensureCursorVisible();
                return;
            }

            if (text.endsWith(s, cs) && QString::compare(s, text, cs) != 0)
                start_pos = cr.selectionStart();
            else
                start_pos = cr.selectionStart() - 1;
        }
    }
    else
    {
        start_pos = cr.position();
    }

    QTextCursor res;

    if (forward)
    {
        res = impl->text->document()->find(text, start_pos, flags);
        if (res.isNull())
            res = impl->text->document()->find(text, 0, flags);
    }
    else
    {
        if (start_pos >= 0)
            res = impl->text->document()->find(text, start_pos, flags);
        if (res.isNull())
        {
            QTextCursor tmp(impl->text->document());
            tmp.movePosition(QTextCursor::End);
            res = impl->text->document()->find(text, tmp.position(), flags);
        }
    }

    if (res.isNull())
    {
        res = impl->text->textCursor();
        res.clearSelection();
        impl->text->setTextCursor(res);
//         QApplication::beep();
    }
    else
    {
        impl->text->setTextCursor(res);
        impl->text->ensureCursorVisible();
    }
}

void SearchBox::entryTextEdited(const QString &text)
{
    doFind(text, true);
}

void SearchBox::findNext()
{
    QString text = impl->searchEntry->text();
    if (text.isEmpty())
        return;
    doFind(text, true);
}

void SearchBox::findPrevious()
{
    QString text = impl->searchEntry->text();
    if (text.isEmpty())
        return;
    doFind(text, false);
}

bool SearchBox::eventFilter(QObject *obj, QEvent *event)
{
    if (obj != impl->searchEntry || event->type() != QEvent::KeyPress)
        return QFrame::eventFilter(obj, event);

    QKeyEvent *ke = static_cast<QKeyEvent*>(event);
    switch (ke->key())
    {
        case Qt::Key_Escape:
            cancelFind();
            return true;
        case Qt::Key_Return:
        case Qt::Key_Enter:
            findNext();
            return true;
        default:
            return QFrame::eventFilter(obj, event);
    }
}
